package asd.graphe;

import asd.base.FileStatique;

/**
 * Une classe pour le parcours en largeur d'un graphe orient ou non.
 * (voir le <a href="../../../html/PL.html">source</a>).
 * @author  2002 Marc Gatano
 * @version 2.0 (02/12/02)
 */
public class PL {

    private Graphe G; // le graphe
    private int[] c;  // c[i] = la couleur du sommet i

    private int[] D;  // D[i] = la distance min de s  i o s est la racine
    private int[] p;  // p[v] = u <==> u est le pre de v

    private static final int blanc = 0;
    private static final int gris  = 1;
    private static final int noir  = 2;

    public PL(Graphe G) {
	this.G = G;
	this.c = new int[G.ordre()];
	this.D = new int[G.ordre()];
	this.p = new int[G.ordre()];
    }

    /**
     * Parcours le graphe G en largeur  partir du sommet s (la racine) et
     * retourne un tableau D tel que D[i] est la distance (en nombre d'arcs ou
     * d'artes) minimale pour aller de s  i, pour tout i dans [0..n-1] o
     * n est l'ordre du graphe.
     * Complexit : O(S + A) o S et A sont respectivement le nombre de
     * sommets et d'artes de G.
     */
    public int[] parcourir(int s) {
	for( int i = 0; i < G.ordre(); i++ ){
		c[i] = blanc;
		D[i] = -1;
		p[i] = -1;
	}

	FileStatique f = new FileStatique(G.ordre());
	c[s] = gris;
	D[s] = 0;

	ajouter(s,f);

	// - u et v sont frres <==> p[u] = p[v]
        // - si (u,v) est un arc, delta(s,v) <= delta(s,u) + 1

        // INVARIANT :
        // - u est gris <==> u est dans f
	// - u est gris ou noir <==> D[u] = delta(s,u)
	// - si u est gris ou noir et v blanc alors delta(s,v) > delta(s,u)
        // - si u est blanc et accessible depuis s alors il existe un
        //   chemin constitu de sommets blancs de w  u avec w gris
        // VARIANT :
        // le couple (nB, |f|) diminue strictement (ordre lexicographique)
        // o nB est le nombre de sommets blancs et |f| la longueur de f
        //
	while ( nonVide(f) ) {

	    int u = enlever(f);        // u est gris

	    Adjacents adj = G.adjacents(u);

	    while ( adj.nonVide() ) {
		int v = adj.suivant();
		if ( c[v] == blanc ) { // si v est blanc alors v n'est pas un
		    c[v] = gris;       // frre de u.

		    D[v] = D[u] + 1;   // delta(s,v) <= delta(s,u) + 1 et
		    p[v] = u;          // delta(s,v) > delta(s,u) donc
		                       // delta(s,v) = delta(s,u) + 1
		    ajouter(v,f);
		}
	    }

	    c[u] = noir;
	}
        return D;
    }

    // pour se simplifier la vie

    private boolean nonVide(FileStatique f) {
        return ! f.vide();
    }

    private void ajouter(int i, FileStatique f) {
        f.ajouter(new Integer(i));
    }

    private int enlever(FileStatique f) {
        int i = ((Integer) f.suivant()).intValue();
        f.retirer();
        return i;
    }
}
